<?php 

/** 
* PHP class to connect the msv3-interface of Gebrauchs.Info
* for full usage of trading operations of pharmaceutical wholesale
* 
* @package msv3client 
* @author Kai Steffens
* @version 3.1.2 - (for PHP5, PHP7, PHP8)
* @access public 
* @see https://www.gebrauchs.info/developer
*/ 

namespace Gebrauchsinfo\Msv3;

class msv3client {
	
	private $auth_token = null;
		
	private $api_url  = 'https://msv3.gebrauchs.info/interface/';
	
	private $api_user = null;
	
	private $api_pass = null;	
	
	private $api_mode = null; // sandbox, live
	
	private $msv3_username = null; 

	private $msv3_password = null;

	private $provider = null; // anzag, gehe, noweda, phoenix, sanacorp, spangropharm

	private $branchnumber = null;
	
	public $last_http_code = null;
	
	public function __construct($api_mode, $api_user, $api_pass, $msv3_username, $msv3_password, $provider, $branchnumber = '01') {
		
		$valid_modes = array('sandbox', 'live');

		if(in_array($api_mode, $valid_modes)) {
			
			$this->api_mode = $api_mode;
		}
		else {
			
			throw new \Exception('unknown api-mode given - allowed values: '.implode(', ', $valid_modes));
		}
		
		foreach(explode(' ', 'api_mode api_user api_pass msv3_username msv3_password provider branchnumber') as $varName) {
			
			if(empty(${$varName})) {
			
				throw new \Exception('no '.$varName.' given');
			}
			else {
				
				$this->{$varName} = ${"$varName"};
			}
		}
		
		$this->getAuthToken();
	}
	
	private function request($func, $postData) {

		$curlServiceUrl = $this->api_url.$func;

		$ch = curl_init($curlServiceUrl);
		curl_setopt($ch, CURLOPT_VERBOSE, 1);

		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);

		curl_setopt($ch, CURLOPT_SSLVERSION, 'CURL_SSLVERSION_TLSv1_2');
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

		if(isset($postData) && count($postData) > 0) {
			
			$curlPostData = http_build_query($postData);
			
			curl_setopt($ch, CURLOPT_POST, true);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $curlPostData);
		}

		$response = curl_exec($ch);

		$http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);

		curl_close($ch); 		

		$this->last_http_code = $http_code;
			
		return intval($http_code) >= 400 ? false : json_decode($response, true);
	}
	
	public function getAuthToken($force_refresh = false) {
	
		if(empty($this->auth_token) || $force_refresh) {			

			$this->auth_token = $this->request('getAuthToken', array('username' => $this->api_user, 'password' => $this->api_pass));
		}
		
		if(empty($this->auth_token)) {
			
			throw new \Exception('authentifaction failed - no valid auth_token received');
		}
		
		return $this->auth_token;
	}	
	
	/**
	* 1.test-step: test connection to msv3-service of gebrauchs.info
	*/
	public function test($testStr = '') {

		$auth_token = $this->getAuthToken();
	
		$postData = array(	'auth_token' => $auth_token, 
							'msv3_username' => $this->msv3_username, 
							'msv3_password' => $this->msv3_password, 
							'mode' => $this->api_mode, 
							'provider' => $this->provider, 
							'branchnumber' => $this->branchnumber,
							'testStr' => $testStr
					);

		return $this->request('test', $postData);
	}	
	
	/**
	* 2.test-step: test connection to msv3-service of pharmaceutical wholesale
	*/
	public function verbindungTesten() {

		$auth_token = $this->getAuthToken();
	
		$postData = array(	'auth_token' => $auth_token, 
							'msv3_username' => $this->msv3_username, 
							'msv3_password' => $this->msv3_password, 
							'mode' => $this->api_mode, 
							'provider' => $this->provider, 
							'branchnumber' => $this->branchnumber
					);

		return $this->request('verbindungTesten', $postData);
	}		
	
	/**
	* get full pzn list of available articles (unqualified response)
	*/
	public function verfuegbarkeitAnfragenBulk() {
		
		$auth_token = $this->getAuthToken();
		
		$postData = array(	'auth_token' => $auth_token, 
							'msv3_username' => $this->msv3_username, 
							'msv3_password' => $this->msv3_password, 
							'mode' => $this->api_mode, 
							'provider' => $this->provider, 
							'branchnumber' => $this->branchnumber
					);

		return $this->request('verfuegbarkeitAnfragenBulk', $postData);
	}
	
	/** 
	* get qualified availability response for single pzn and amount
	*/
	public function verfuegbarkeitAnfragen($pzn, $amount) {

		$auth_token = $this->getAuthToken();

		$postData = array(	'auth_token' => $auth_token, 
							'msv3_username' => $this->msv3_username, 
							'msv3_password' => $this->msv3_password, 
							'mode' => $this->api_mode, 
							'provider' => $this->provider, 
							'branchnumber' => $this->branchnumber,
							'pzn' => $pzn, 
							'amount' => $amount
					);

		return $this->request('verfuegbarkeitAnfragen', $postData);
	}
		
	public static function uuid() {
		
		return sprintf('%04x%04x-%04x-%04x-%04x-%04x%04x%04x',
			mt_rand(0, 0xffff), mt_rand(0, 0xffff),
			mt_rand(0, 0xffff),
			mt_rand(0, 0x0fff) | 0x4000,
			mt_rand(0, 0x3fff) | 0x8000,
			mt_rand(0, 0xffff), mt_rand(0, 0xffff), mt_rand(0, 0xffff)
		);
	}
	
	public function bestellen($basket) {

		$auth_token = $this->getAuthToken();
	
		$postData = array(	'auth_token' => $auth_token, 
							'msv3_username' => $this->msv3_username, 
							'msv3_password' => $this->msv3_password, 
							'mode' => $this->api_mode, 
							'provider' => $this->provider, 
							'branchnumber' => $this->branchnumber,
							'basket' => $basket
					);

		return $this->request('bestellen', $postData);
	}
	
	public function bestellstatusAbfragen($bestellid) {

		$auth_token = $this->getAuthToken();

		$postData = array(	'auth_token' => $auth_token, 
							'msv3_username' => $this->msv3_username, 
							'msv3_password' => $this->msv3_password, 
							'mode' => $this->api_mode, 
							'provider' => $this->provider, 
							'branchnumber' => $this->branchnumber,
							'bestellid' => $bestellid
					);

		return $this->request('bestellstatusAbfragen', $postData);
	}
	
	public function dokumentAbfragen($dokumentId) {

		$auth_token = $this->getAuthToken();

		$postData = array(	'auth_token' => $auth_token, 
							'msv3_username' => $this->msv3_username, 
							'msv3_password' => $this->msv3_password, 
							'mode' => $this->api_mode, 
							'provider' => $this->provider, 
							'branchnumber' => $this->branchnumber,
							'dokumentId' => $dokumentId
					);

		return $this->request('dokumentAbfragen', $postData);
	}
	
	public function ruecknahmeangebotAnfordern($articles) {

		$auth_token = $this->getAuthToken();

		$postData = array(	'auth_token' => $auth_token, 
							'msv3_username' => $this->msv3_username, 
							'msv3_password' => $this->msv3_password, 
							'mode' => $this->api_mode, 
							'provider' => $this->provider, 
							'branchnumber' => $this->branchnumber,
							'articles' => $articles
					);

		return $this->request('ruecknahmeangebotAnfordern', $postData);
	}	
	
	public function lieferavisAbfrageNeu() {

		$auth_token = $this->getAuthToken();

		$postData = array(	'auth_token' => $auth_token, 
							'msv3_username' => $this->msv3_username, 
							'msv3_password' => $this->msv3_password, 
							'mode' => $this->api_mode, 
							'provider' => $this->provider, 
							'branchnumber' => $this->branchnumber
					);

		return $this->request('lieferavisAbfrageNeu', $postData);
	}		
	
	public function lieferavisBestaetigen($lieferscheinnummer) {

		$auth_token = $this->getAuthToken();

		$postData = array(	'auth_token' => $auth_token, 
							'msv3_username' => $this->msv3_username, 
							'msv3_password' => $this->msv3_password, 
							'mode' => $this->api_mode, 
							'provider' => $this->provider, 
							'branchnumber' => $this->branchnumber,
							'lieferscheinnummer' => $lieferscheinnummer
					);

		return $this->request('lieferavisBestaetigen', $postData);
	}
	
	public function lieferavisAbfrageHist($lieferscheinnummer, $barcodeReferenz = null) {

		$auth_token = $this->getAuthToken();

		$postData = array(	'auth_token' => $auth_token, 
							'msv3_username' => $this->msv3_username, 
							'msv3_password' => $this->msv3_password, 
							'mode' => $this->api_mode, 
							'provider' => $this->provider, 
							'branchnumber' => $this->branchnumber,
							'lieferscheinnummer' => $lieferscheinnummer,							
					);

		if(!empty($barcodeReferenz)) {
			
			$postData['barcodeReferenz'] = $barcodeReferenz;
		}

		return $this->request('lieferavisAbfrageHist', $postData);
	}	
	
	public function retourenavisAnfragen($positionen, $retoureSupportID) {

		$auth_token = $this->getAuthToken();

		$postData = array(	'auth_token' => $auth_token, 
							'msv3_username' => $this->msv3_username, 
							'msv3_password' => $this->msv3_password, 
							'mode' => $this->api_mode, 
							'provider' => $this->provider, 
							'branchnumber' => $this->branchnumber,
							'positionen' => $positionen,
							'retoureSupportID' => $retoureSupportID
					);

		return $this->request('retourenavisAnfragen', $postData);
	}	
	
	public function retourenavisAnkuendigen($positionen, $retoureSupportID, $kopfdaten) {

		$auth_token = $this->getAuthToken();

		$postData = array(	'auth_token' => $auth_token, 
							'msv3_username' => $this->msv3_username, 
							'msv3_password' => $this->msv3_password, 
							'mode' => $this->api_mode, 
							'provider' => $this->provider, 
							'branchnumber' => $this->branchnumber,
							'positionen' => $positionen,
							'retoureSupportID' => $retoureSupportID,
							'kopfdaten' => $kopfdaten
					);

		return $this->request('retourenavisAnkuendigen', $postData);
	}	
}